@@ -57,13 +57,13 @@ struct _FindOptionTemplate {
 };
 	
 static FindOptionTemplate templates[] = {
-	{ SEARCH_CONSTRAINT_TEXT, "-exec grep -q '%s' {} \\;", N_("Contains the text"), FALSE },
+	{ SEARCH_CONSTRAINT_TEXT, "-exec grep -c '%s' {} \\;", N_("Contains the text"), FALSE },
 	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
 	{ SEARCH_CONSTRAINT_TIME, "-mtime -%d", N_("Date modified less than (days)"), FALSE },
 	{ SEARCH_CONSTRAINT_TIME, "-mtime +%d", N_("Date modified more than (days)"), FALSE },
 	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
-	{ SEARCH_CONSTRAINT_NUMBER, "-size +%dk", N_("Size at least (kilobytes)"), FALSE }, 
-	{ SEARCH_CONSTRAINT_NUMBER, "-size -%dk", N_("Size at most (kilobytes)"), FALSE },
+	{ SEARCH_CONSTRAINT_NUMBER, "-size +%uc", N_("Size at least (kilobytes)"), FALSE }, 
+	{ SEARCH_CONSTRAINT_NUMBER, "-size -%uc", N_("Size at most (kilobytes)"), FALSE },
 	{ SEARCH_CONSTRAINT_BOOL, "-size 0c \\( -type f -o -type d \\)", N_("File is empty"), FALSE },	
 	{ SEARCH_CONSTRAINT_SEPARATOR, NULL, NULL, TRUE },
 	{ SEARCH_CONSTRAINT_TEXT, "-user '%s'", N_("Owned by user"), FALSE },
@@ -184,18 +184,19 @@ build_search_command (void)
 		file_is_named_locale = NULL;
 	} 
 	else {
-		file_is_named_locale = g_filename_from_utf8 (file_is_named_utf8, -1, NULL, NULL, NULL);
+		file_is_named_locale = g_locale_from_utf8 (file_is_named_utf8, -1, NULL, NULL, NULL);
 		gnome_entry_prepend_history (GNOME_ENTRY(interface.file_is_named_entry), TRUE, file_is_named_utf8);
 	}
 	
-	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), TRUE);
-	look_in_folder_locale = g_filename_from_utf8 (look_in_folder_utf8, -1, NULL, NULL, NULL);
+	look_in_folder_utf8 = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
+	look_in_folder_locale = g_locale_from_utf8 (look_in_folder_utf8, -1, NULL, NULL, NULL);
 	gnome_entry_prepend_history (GNOME_ENTRY(gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY(interface.look_in_folder_entry))), 
 				     TRUE, look_in_folder_utf8);
 	
 	if (!file_extension_is (look_in_folder_locale, G_DIR_SEPARATOR_S)) {
 		look_in_folder_locale = g_strconcat (look_in_folder_locale, G_DIR_SEPARATOR_S, NULL);
 	}
+	search_command.look_in_folder = g_strdup(look_in_folder_locale);
 	
 	command = g_string_new ("");
 	
@@ -205,7 +206,7 @@ build_search_command (void)
 		
 		locate = g_find_program_in_path ("locate");
 		file_is_named_escaped = escape_single_quotes (file_is_named_locale);
-		search_command.file_is_named_pattern = g_strdup(file_is_named_locale);
+		search_command.file_is_named_pattern = g_strdup(file_is_named_utf8);
 		
 		if ((locate != NULL) && (is_path_in_home_folder (look_in_folder_locale) != TRUE)) {
 			g_string_append_printf (command, "%s '%s*%s'", 
@@ -226,7 +227,7 @@ build_search_command (void)
 		
 		search_command.regex_matching_enabled = FALSE;
 		file_is_named_escaped = escape_single_quotes (file_is_named_locale);
-		search_command.file_is_named_pattern = g_strdup(file_is_named_locale);
+		search_command.file_is_named_pattern = g_strdup(file_is_named_utf8);
 		
 		if (file_is_named_escaped == NULL) {
 			g_string_append_printf (command, "find \"%s\" '!' -type d '!' -type p -name '%s' ", 
@@ -286,7 +287,7 @@ build_search_command (void)
 			case SEARCH_CONSTRAINT_NUMBER:
 				g_string_append_printf (command,
 					  		templates[constraint->constraint_id].option,
-					  		constraint->data.number);
+					  		(constraint->data.number * 1024));
 				g_string_append_c (command, ' ');
 				break;
 			case SEARCH_CONSTRAINT_TIME:
@@ -321,8 +322,8 @@ add_file_to_search_results (const gchar 	*file,
 	gchar *mime_type = gnome_vfs_get_mime_type (file);
 	gchar *description = get_file_type_with_mime_type (file, mime_type);
 	gchar *icon_path = get_file_icon_with_mime_type (file, mime_type);
-	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
 	GnomeVFSFileInfo *vfs_file_info = gnome_vfs_file_info_new ();
+	GdkPixbuf *pixbuf = NULL;
 	gchar *readable_size, *readable_date, *date;
 	gchar *utf8_base_name, *utf8_dir_name;
 	gchar *base_name, *dir_name;
@@ -331,6 +332,10 @@ add_file_to_search_results (const gchar 	*file,
 		gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(interface.tree), TRUE);
 	}
 	
+	if (icon_path != NULL) {
+		pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
+	}
+	
 	if (pixbuf != NULL) {
 		GdkPixbuf *scaled;
 		int        new_w, new_h;
@@ -378,8 +383,11 @@ add_file_to_search_results (const gchar 	*file,
 			    COLUMN_NO_FILES_FOUND, FALSE,
 			    -1);
 
+	if (pixbuf != NULL) {
+		g_object_unref (G_OBJECT(pixbuf));
+	}
+
 	gnome_vfs_file_info_unref (vfs_file_info);
-	g_object_unref (G_OBJECT(pixbuf));
 	g_free (base_name);
 	g_free (dir_name);
 	g_free (mime_type);
@@ -743,23 +751,26 @@ handle_search_command_stdout_io (GIOChannel 	*ioc,
 				continue;
 			}
 			
-			filename = g_path_get_basename (locale);
+			if (strncmp (string->str, search_data->look_in_folder, strlen (search_data->look_in_folder)) == 0) { 
 			
-			if (fnmatch (search_data->file_is_named_pattern, filename, FNM_NOESCAPE) != FNM_NOMATCH) {
-				if (search_data->show_hidden_files == TRUE) {
-					if (search_data->regex_matching_enabled == FALSE) {
-						add_file_to_search_results (string->str, interface.model, &interface.iter);
-					} 
-					else if (compare_regex (search_data->regex_string, filename) == TRUE) {
-						add_file_to_search_results (string->str, interface.model, &interface.iter);
+				filename = g_path_get_basename (locale);
+			
+				if (fnmatch (search_data->file_is_named_pattern, filename, FNM_NOESCAPE) != FNM_NOMATCH) {
+					if (search_data->show_hidden_files == TRUE) {
+						if (search_data->regex_matching_enabled == FALSE) {
+							add_file_to_search_results (string->str, interface.model, &interface.iter);
+						} 
+						else if (compare_regex (search_data->regex_string, filename) == TRUE) {
+							add_file_to_search_results (string->str, interface.model, &interface.iter);
+						}
 					}
-				}
-				else if (is_path_hidden (string->str) == FALSE) {
-					if (search_data->regex_matching_enabled == FALSE) {
-						add_file_to_search_results (string->str, interface.model, &interface.iter);
-					} 
-					else if (compare_regex (search_data->regex_string, filename) == TRUE) {
-						add_file_to_search_results (string->str, interface.model, &interface.iter);
+					else if (is_path_hidden (string->str) == FALSE) {
+						if (search_data->regex_matching_enabled == FALSE) {
+							add_file_to_search_results (string->str, interface.model, &interface.iter);
+						} 
+						else if (compare_regex (search_data->regex_string, filename) == TRUE) {
+							add_file_to_search_results (string->str, interface.model, &interface.iter);
+						}
 					}
 				}
 			}
@@ -1037,7 +1048,7 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 			gchar *desc = g_strconcat (_(templates[opt->constraint_id].desc), ".", NULL);
 			label = gtk_label_new (desc);
 			g_free (desc);
-			gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD);
+			gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
 		}
 		break;
 	case SEARCH_CONSTRAINT_TEXT:
@@ -1048,7 +1059,7 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 			label = gtk_label_new (desc);
 			g_free (desc);
 		}
-		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD);
+		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
 		
 		if (templates[opt->constraint_id].type == SEARCH_CONSTRAINT_TEXT) {
 			entry = gtk_entry_new();
@@ -1083,7 +1094,7 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 				  NULL);
 				  
 		/* add label and text field */
-		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, GNOME_PAD_SMALL);
 		
 		break;
 	default:
@@ -1097,7 +1108,7 @@ create_constraint_box (SearchConstraint *opt, gchar *value)
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(remove_constraint_cb), opt);
         gtk_size_group_add_widget (interface.constraint_size_group, button);
-	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, GNOME_PAD_SMALL);
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	
 	if (interface.is_gail_loaded) {
 		gchar *desc = g_strdup_printf (_("Click to Remove the '%s' Rule"), 
@@ -1146,8 +1157,8 @@ create_additional_constraint_section (void)
 	GtkWidget *label;
 	GtkWidget *button;
 
-	vbox1 = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
-	gtk_container_set_border_width (GTK_CONTAINER(vbox1), GNOME_PAD_SMALL);
+	vbox1 = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER(vbox1), 0);
 
 	interface.constraint = gtk_vbox_new (FALSE, 0); 
 	vbox2 = gtk_vbox_new (FALSE, 0);
@@ -1156,12 +1167,12 @@ create_additional_constraint_section (void)
 	gtk_box_pack_end (GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
 	
 	label = gtk_label_new_with_mnemonic (_("A_vailable options:"));
-	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD);
+	gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD_SMALL);
 	
 	interface.constraint_menu = gtk_option_menu_new ();
 	gtk_label_set_mnemonic_widget (GTK_LABEL(label), GTK_WIDGET(interface.constraint_menu));
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(interface.constraint_menu), make_list_of_templates());
-	gtk_box_pack_start (GTK_BOX(hbox), interface.constraint_menu, TRUE, TRUE, GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX(hbox), interface.constraint_menu, TRUE, TRUE, GNOME_PAD);
 
 	if (interface.is_gail_loaded)
 	{
@@ -1172,15 +1183,15 @@ create_additional_constraint_section (void)
 
 	button = gtk_button_new_from_stock (GTK_STOCK_ADD);
 	interface.constraint_size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-	gtk_widget_set_size_request (GTK_WIDGET(button), 78, -1);
 	gtk_size_group_add_widget (interface.constraint_size_group, button);
+	gtk_size_group_add_widget (interface.constraint_size_group, gtk_button_new_from_stock (GTK_STOCK_REMOVE));
 	
 	g_signal_connect (G_OBJECT(button),"clicked",
 			  G_CALLBACK(add_constraint_cb),NULL);
 	
-	gtk_box_pack_end (GTK_BOX(hbox), button, FALSE, FALSE, GNOME_PAD_SMALL); 
+	gtk_box_pack_end (GTK_BOX(hbox), button, FALSE, FALSE, 0); 
 		
-	gtk_box_pack_start (GTK_BOX(vbox1), interface.constraint, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX(vbox1), interface.constraint, TRUE, TRUE, GNOME_PAD_SMALL);
 	
 	gtk_container_add (GTK_CONTAINER(vbox1), GTK_WIDGET(vbox2));
 	
@@ -1206,7 +1217,7 @@ create_search_results_section (void)
 	interface.results = gtk_scrolled_window_new (NULL, NULL);
 	gtk_widget_set_sensitive (GTK_WIDGET(interface.results), FALSE); 
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(interface.results), GTK_SHADOW_IN);	
-	gtk_container_set_border_width (GTK_CONTAINER(interface.results), GNOME_PAD_SMALL);
+	gtk_container_set_border_width (GTK_CONTAINER(interface.results), 0);
 	gtk_widget_set_usize (interface.results, 530, 200); 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(interface.results),
                                         GTK_POLICY_AUTOMATIC,
@@ -1437,6 +1448,9 @@ create_main_window (void)
 
 	gtk_box_pack_start (GTK_BOX(window), GTK_WIDGET(interface.additional_constraints), FALSE, FALSE, 0);
 	
+	results = create_search_results_section ();
+	gtk_box_pack_start (GTK_BOX(window), results, TRUE, TRUE, GNOME_PAD_SMALL);
+	
 	hsep = gtk_hseparator_new ();
 	gtk_box_pack_start (GTK_BOX(window),hsep,FALSE,FALSE,0);
 	
@@ -1475,9 +1489,6 @@ create_main_window (void)
 		add_atk_namedesc (GTK_WIDGET(interface.stop_button), NULL, _("Click to Stop the search"));
 	}
 
-	results = create_search_results_section ();
-	gtk_box_pack_start (GTK_BOX(window), results, TRUE, TRUE, GNOME_PAD_SMALL);
-
 	return window;
 }
 
@@ -1535,7 +1546,7 @@ main (int 	argc,
 	gtk_window_set_geometry_hints (GTK_WINDOW(interface.main_window), GTK_WIDGET(interface.main_window),
 				       &interface.geometry, GDK_HINT_MIN_SIZE);
 
-	gtk_window_set_type_hint (GTK_WINDOW(interface.main_window),GDK_WINDOW_TYPE_HINT_DIALOG);
+	gtk_window_set_position (GTK_WINDOW(interface.main_window), GTK_WIN_POS_CENTER);
 
 	gtk_window_set_focus (GTK_WINDOW(interface.main_window), 
 		GTK_WIDGET(gnome_entry_gtk_entry(GNOME_ENTRY(interface.file_is_named_entry))));
