@@ -37,49 +37,45 @@
  *    -------------------
  */
 
-void repaint (GtkWidget * canvas, GdkRectangle * area);
 void CreateMainWin (void);
-gboolean log_repaint (GtkWidget * canvas, GdkRectangle * area);
-gboolean PointerMoved (GtkWidget * canvas, GdkEventMotion * event);
-gboolean HandleLogKeyboard (GtkWidget * win, GdkEventKey * event_key);
-gboolean handle_log_mouse_button (GtkWidget * win, GdkEventButton *event);
-gboolean handle_log_mouse_scroll (GtkWidget * win, GdkEventScroll *event);
+gboolean log_repaint (void);
+gboolean handle_log_mouse_button (GtkWidget *view, GdkEventButton *event);
 void ExitProg (GtkWidget * widget, gpointer user_data);
 void LoadLogMenu (GtkWidget * widget, gpointer user_data);
 void CloseLogMenu (GtkWidget * widget, gpointer user_data);
 void change_log_menu (GtkWidget * widget, gpointer user_data);
 void CalendarMenu (GtkWidget * widget, gpointer user_data);
 void MonitorMenu (GtkWidget* widget, gpointer user_data); 
-void create_zoom_view (GtkWidget *widget, gpointer data);
-void UserPrefsDialog(GtkWidget * widget, gpointer user_data);
+void create_zoom_view (void);
 void AboutShowWindow (GtkWidget* widget, gpointer user_data);
 void CloseApp (void);
 void CloseLog (Log *);
 void FileSelectCancel (GtkWidget * w, GtkFileSelection * fs);
 void FileSelectOk (GtkWidget * w, GtkFileSelection * fs);
-void MainWinScrolled (GtkAdjustment *adjustment, GtkRange *);
-void CanvasResized (GtkWidget *widget, GtkAllocation *allocation);
-gboolean ScrollWin (GtkRange *range, gpointer event);
 void LogInfo (GtkWidget * widget, gpointer user_data);
 void UpdateStatusArea (void);
-void set_scrollbar_size (int);
 void change_log (int dir);
 void open_databases (void);
 void destroy (void);
 void InitApp (void);
 int InitPages (void);
-int RepaintLogInfo (GtkWidget * widget, GdkEventExpose * event);
+int RepaintLogInfo (void);
 int read_regexp_db (char *filename, GList **db);
 int read_actions_db (char *filename, GList **db);
 void print_db (GList *gb);
 Log *OpenLogFile (char *);
 GtkWidget *new_pixmap_from_data (char  **, GdkWindow *, GdkColor *);
-GtkWidget *create_menu (char *item[], int n);
 void SaveUserPrefs(UserPrefsStruct *prefs);
 void close_zoom_view (GtkWidget *widget, gpointer client_data);
+void handle_selection_changed_cb (GtkTreeSelection *selection, gpointer data);
+void handle_row_activation_cb (GtkTreeView *treeview, GtkTreePath *path, 
+     GtkTreeViewColumn *arg2, gpointer user_data);
+void save_rows_to_expand (Log *log);
+void CloseAllLogs (void);
 
 static void toggle_calendar (void);
 static void toggle_zoom (void);
+static void toggle_collapse_rows (void);
 
 /*
  *    ,-------.
@@ -87,52 +83,92 @@ static void toggle_zoom (void);
  *    `-------'
  */
 
+enum {
+	MENU_FILE_OPEN = 0,
+	MENU_SWITCH_LOG = 2,
+	MENU_MONITOR = 3,
+	MENU_PROPERTIES = 5,
+	MENU_CLOSE = 7,
+	MENU_CLOSE_ALL = 8,
+	MENU_QUIT = 9 
+};
 
 GnomeUIInfo log_menu[] = {
 	GNOMEUIINFO_MENU_OPEN_ITEM(LoadLogMenu, NULL),
-	GNOMEUIINFO_MENU_SAVE_AS_ITEM(StubCall, NULL),
 	GNOMEUIINFO_SEPARATOR,
-        { GNOME_APP_UI_ITEM, N_("S_witch Log"), 
-	  N_("Switch log"), change_log_menu, NULL, NULL,
-          GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
-        { GNOME_APP_UI_ITEM, N_("_Monitor..."), 
-	  N_("Monitor Log"), MonitorMenu, NULL, NULL,
-          GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
+    { GNOME_APP_UI_ITEM, N_("S_witch Log"),
+			 N_("Switch between already opened logs"),
+			 change_log_menu,
+			 NULL, NULL,
+			 GNOME_APP_PIXMAP_NONE, NULL,
+			 'N', GDK_CONTROL_MASK, NULL },
+
+    { GNOME_APP_UI_ITEM, N_("_Monitor..."),
+			 N_("Monitor Logs"),
+		         MonitorMenu,
+			 NULL,
+			 NULL,
+			 GNOME_APP_PIXMAP_NONE, NULL,
+			 'M', GDK_CONTROL_MASK, NULL },
+ 
 	GNOMEUIINFO_SEPARATOR,
-        { GNOME_APP_UI_ITEM, N_("_Properties"), 
-	  N_("Show Log Properties"), LogInfo, NULL, NULL,
-          GNOME_APP_PIXMAP_NONE, NULL, 'I', GDK_CONTROL_MASK, NULL},
+    { GNOME_APP_UI_ITEM, N_("_Properties"), 
+				N_("Show Log Properties"), 
+				LogInfo, 
+				NULL, 
+				NULL,
+				GNOME_APP_PIXMAP_NONE, NULL, 
+				'I', GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_CLOSE_ITEM(CloseLogMenu, NULL),
-	GNOMEUIINFO_MENU_QUIT_ITEM(ExitProg, NULL),
-        {GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL}
+	GNOMEUIINFO_MENU_CLOSE_ITEM (CloseLogMenu, NULL),
+    { GNOME_APP_UI_ITEM, N_("Clos_e All"),
+				N_("Close all Log files"), 
+				CloseAllLogs, 
+				NULL, 
+				NULL,
+				GNOME_APP_PIXMAP_STOCK, 
+				GNOME_STOCK_MENU_CLOSE, 
+				'W', GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
+	GNOMEUIINFO_MENU_QUIT_ITEM (ExitProg, NULL),
+    GNOMEUIINFO_END
 };
 
 GnomeUIInfo view_menu[] = {
-        { GNOME_APP_UI_TOGGLEITEM, N_("_Calendar"), N_("Show Calendar Log"), toggle_calendar, 
-	  NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'L', GDK_CONTROL_MASK, NULL },
-        { GNOME_APP_UI_TOGGLEITEM, N_("_Entry Detail"), N_("Show Entry Detail"), toggle_zoom, 
-	  NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'D', GDK_CONTROL_MASK, NULL },
-        {GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL}
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Calendar"),
+				N_("Show Calendar Log"), 
+				toggle_calendar, 
+				NULL, 
+				NULL,
+				GNOME_APP_PIXMAP_NONE, NULL, 
+				'L', GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Entry Detail"), 
+				N_("Show Entry Detail"), 
+				toggle_zoom, 
+				NULL, 
+				NULL, 
+				GNOME_APP_PIXMAP_NONE, NULL, 
+				'D', GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Collapse All"),
+				N_("Collapse all the rows"),
+				toggle_collapse_rows,
+				NULL,
+				NULL,
+				GNOME_APP_PIXMAP_NONE, NULL, 
+				'C', GDK_CONTROL_MASK, NULL },
+	GNOMEUIINFO_END
 };
 
-
 GnomeUIInfo help_menu[] = {
-	GNOMEUIINFO_HELP("gnome-system-log"),
-        {GNOME_APP_UI_ITEM, N_("About..."), 
-	 N_("Info about logview"), AboutShowWindow,
-         NULL, NULL,
-         GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL},
-        {GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL}
+      GNOMEUIINFO_HELP ("gnome-system-log"),
+      GNOMEUIINFO_MENU_ABOUT_ITEM (AboutShowWindow, NULL),
+      GNOMEUIINFO_END
 };
 
 GnomeUIInfo main_menu[] = {
-        { GNOME_APP_UI_SUBTREE, N_("_Log"), NULL,
-          log_menu, NULL, NULL, (GnomeUIPixmapType) 0,
-          NULL, 0, (GdkModifierType) 0, NULL },
-	GNOMEUIINFO_MENU_VIEW_TREE(view_menu),
-	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
-        {GNOME_APP_UI_ENDOFINFO, NULL, NULL, NULL}
+	GNOMEUIINFO_MENU_FILE_TREE (log_menu),
+	GNOMEUIINFO_MENU_VIEW_TREE (view_menu),
+	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
+	GNOMEUIINFO_END
 };
                  
 
@@ -141,19 +177,17 @@ GnomeUIInfo main_menu[] = {
  *       Global variables
  *       ----------------
  */
-
-
+	
 GtkWidget *app = NULL;
-GtkWidget *main_win_scrollbar = NULL;
 static GtkWidget *viewport;
-GtkWidget *main_win_hor_scrollbar = NULL; 
-GtkLabel *filename_label = NULL, *date_label = NULL;
+GtkLabel *date_label = NULL;
 
 GList *regexp_db = NULL, *descript_db = NULL, *actions_db = NULL;
 UserPrefsStruct *user_prefs = NULL;
 UserPrefsStruct user_prefs_struct = {0};
 ConfigData *cfg = NULL;
 GtkWidget *open_file_dialog = NULL;
+GtkWidget *view = NULL;
 
 GConfClient *client;
 poptContext poptCon;
@@ -171,13 +205,11 @@ struct poptOption options[] = { {
 
 extern GtkWidget *CalendarDialog;
 extern GtkWidget *zoom_dialog;
-extern GdkGC *gc;
 extern Log *curlog, *loglist[];
 extern int numlogs, curlognum;
 extern int loginfovisible, calendarvisible;
 extern int cursor_visible;
 extern int zoom_visible;
-extern PangoLayout *log_layout;
 
 /* ----------------------------------------------------------------------
    NAME:          destroy
@@ -210,6 +242,7 @@ save_session (GnomeClient *gclient, gint phase,
    g_free (argv);
 
    return TRUE;
+
 }
 
 static gboolean
@@ -223,10 +256,9 @@ restore_session (void)
    curlog = NULL;
    loglist[0] = NULL;
    curlognum = 0;
-   log_repaint (NULL, NULL);
+   log_repaint ();
    if (loginfovisible)
-       RepaintLogInfo (NULL, NULL);
-   set_scrollbar_size (1);
+       RepaintLogInfo ();
    numlogs = 0;
 
    next_opt = poptGetNextOpt (poptCon);
@@ -262,51 +294,51 @@ die (GnomeClient *gclient, gpointer client_data)
 int
 main (int argc, char *argv[])
 {
-  GnomeClient *gclient;
+   GnomeClient *gclient;
 
-  bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
-  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
-  textdomain(GETTEXT_PACKAGE);
+   bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+   bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+   textdomain(GETTEXT_PACKAGE);
 
-  QueueErrMessages (TRUE);
+   QueueErrMessages (TRUE);
 
-  /*  Initialize gnome & gtk */
-  gnome_program_init ("gnome-system-log",VERSION, LIBGNOMEUI_MODULE, argc, argv,
-  		      GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
+   /*  Initialize gnome & gtk */
+   gnome_program_init ("gnome-system-log",VERSION, LIBGNOMEUI_MODULE, argc, argv,
+			   GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
-  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-log.png");
-  
-  poptCon = poptGetContext ("gnome-system-log", argc, (const gchar **) argv, 
-                            options, 0);  
-  gclient = gnome_master_client ();
-  g_signal_connect (gclient, "save_yourself",
-                    G_CALLBACK (save_session), (gpointer)argv[0]);
-  g_signal_connect (gclient, "die", G_CALLBACK (die), NULL);
-
-  gconf_init (argc, argv, NULL);
-  client = gconf_client_get_default ();
-  
-  /*  Load graphics config */
-  cfg = CreateConfig();
-  
-  /*  Show about window */
-  /* AboutShowWindow (NULL, NULL); */
-
-  InitApp ();
-
-  QueueErrMessages (FALSE);
-  ShowQueuedErrMessages ();
+   gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-log.png");
    
-  if (gnome_client_get_flags (gclient) & GNOME_CLIENT_RESTORED) {
-     restore_session ();
-  }
+   poptCon = poptGetContext ("gnome-system-log", argc, (const gchar **) argv, 
+							 options, 0);  
+   gclient = gnome_master_client ();
+   g_signal_connect (gclient, "save_yourself",
+					 G_CALLBACK (save_session), (gpointer)argv[0]);
+   g_signal_connect (gclient, "die", G_CALLBACK (die), NULL);
 
-  /*  Loop application */
-  gtk_main ();
-  
-  SaveUserPrefs(user_prefs);
+   gconf_init (argc, argv, NULL);
+   client = gconf_client_get_default ();
+   
+   /*  Load graphics config */
+   cfg = CreateConfig();
+   
+   InitApp ();
+
+   log_repaint (); 
+
+   QueueErrMessages (FALSE);
+   ShowQueuedErrMessages ();
+   
+   if (gnome_client_get_flags (gclient) & GNOME_CLIENT_RESTORED) {
+	  restore_session ();
+   }
+
+   /*  Loop application */
+   gtk_main ();
+   
+   SaveUserPrefs(user_prefs);
+
+   return 0;
 
-  return 0;
 }
 
 /* ----------------------------------------------------------------------
@@ -317,21 +349,21 @@ main (int argc, char *argv[])
 void
 InitApp ()
 {
-  /*  Initialize variables */
-  loginfovisible = FALSE;
-  regexp_db = NULL;
-  user_prefs = &user_prefs_struct;
-  SetDefaultUserPrefs(user_prefs);
+   loginfovisible = FALSE;
+   regexp_db = NULL;
+   user_prefs = &user_prefs_struct;
+   SetDefaultUserPrefs(user_prefs);
 
-  /*  Display main window */
-  CreateMainWin ();
+   /*  Display main window */
+   CreateMainWin ();
 
-  /* Read databases */
-  open_databases ();
+   /* Read databases */
+   open_databases ();
+
+   /*  Read files and init data. */
+   if (InitPages () < 0)
+	 ShowErrMessage (_("No log files to open"));
 
-  /*  Read files and init data. */
-  if (InitPages () < 0)
-    ShowErrMessage (_("No log files to open"));
 }
 
 /* ----------------------------------------------------------------------
@@ -342,16 +374,29 @@ InitApp ()
 void
 CreateMainWin ()
 {
-   GtkWidget *canvas;
    GtkWidget *w, *vbox, *table, *hbox, *hbox_date;
    GtkWidget *padding;
    GtkLabel *label;
    GtkObject *adj;
    GtkAllocation req_size;
+   GtkTreeStore *tree_store;
+   GtkTreeSelection *selection;
+   GtkTreeViewColumn *column;
+   GtkCellRenderer *renderer;
+   GtkWidget *scrolled_window = NULL;
+   gint i;
+   gchar *window_title;
+   const gchar *column_titles[] = { N_("Date"), N_("Host Name"),
+                                    N_("Process"), N_("Message"), NULL };
 
    /* Create App */
 
-   app = gnome_app_new ("gnome-system-log", _("System Log Viewer"));
+   if ((curlog != NULL) && (numlogs))
+       window_title = g_strdup_printf ("%s - %s", curlog->name, APP_NAME);
+   else
+       window_title = g_strdup_printf (APP_NAME);
+   app = gnome_app_new ("gnome-system-log", window_title);
+   g_free (window_title);
 
    gtk_container_set_border_width ( GTK_CONTAINER (app), 0);
    gtk_window_set_default_size ( GTK_WINDOW (app), LOG_CANVAS_W, LOG_CANVAS_H);
@@ -368,98 +413,54 @@ CreateMainWin ()
    vbox = gtk_vbox_new (FALSE, 0);
    gnome_app_set_contents (GNOME_APP (app), vbox);
 
-   /* Deactivate unfinished items */
-   gtk_widget_set_state (log_menu[1].widget, GTK_STATE_INSENSITIVE);
    if (numlogs < 2)
-     gtk_widget_set_state (log_menu[3].widget, GTK_STATE_INSENSITIVE);
+     gtk_widget_set_state (log_menu[MENU_SWITCH_LOG].widget, GTK_STATE_INSENSITIVE);
 
-   /* Create main canvas and scroll bars */
-   table = gtk_table_new (2, 2, FALSE);
-   gtk_widget_show (table);
+   /* Create scrolled window and tree view */
+   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+   gtk_widget_set_sensitive (scrolled_window, TRUE);
+   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+               GTK_POLICY_AUTOMATIC,
+               GTK_POLICY_AUTOMATIC);
+   gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0);
 
-   viewport = gtk_viewport_new (NULL, NULL);
-   gtk_widget_set_size_request (viewport, LOG_CANVAS_W, 0); 
-   gtk_widget_show (viewport);
-               
-   canvas = gtk_drawing_area_new ();
-   gtk_drawing_area_size (GTK_DRAWING_AREA (canvas), 2*LOG_CANVAS_W,
-			  LOG_CANVAS_H); 
-   gtk_container_add (GTK_CONTAINER (viewport), canvas);
-   gtk_table_attach (GTK_TABLE (table), viewport, 0, 1, 0, 1,
-		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+   /* Create Tree View */
+   tree_store = gtk_tree_store_new (4,
+                G_TYPE_STRING, G_TYPE_STRING,
+                G_TYPE_STRING, G_TYPE_STRING);
 
-   if (curlog != NULL)
-	   adj = (GtkObject *)gtk_viewport_get_hadjustment (GTK_VIEWPORT (viewport));
-   else
-	   adj = gtk_adjustment_new (100.0, 0.0, 101.0, 1, 10, 101.0);
+   view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
+   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view), TRUE);
+   g_object_unref (G_OBJECT (tree_store)); 
    
-   main_win_hor_scrollbar = gtk_hscrollbar_new (GTK_ADJUSTMENT (adj));
-   gtk_widget_show(main_win_hor_scrollbar);
-   gtk_table_attach (GTK_TABLE (table), main_win_hor_scrollbar, 0, 1, 1, 2,
-		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-		     (GtkAttachOptions) (0), 0, 0);
+   /* Add Tree View Columns */
+   for (i = 0; column_titles[i]; i++) {
+        renderer = gtk_cell_renderer_text_new ();
+        column = gtk_tree_view_column_new_with_attributes (column_titles[i],
+                    renderer, "text", i, NULL);
+        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE); 
+        gtk_tree_view_column_set_resizable (column, TRUE);
+        gtk_tree_view_column_set_spacing (column, GNOME_PAD_SMALL);
+        gtk_tree_view_append_column (GTK_TREE_VIEW (view), column);
+   }
 
-   if (curlog != NULL)
-     adj = gtk_adjustment_new ( curlog->ln, 0.0,
-				curlog->lstats.numlines,
-				1.0, 10.0, (float) LINES_P_PAGE);
-   else
-     adj = gtk_adjustment_new (100.0, 0.0, 101.0, 1, 10, 101.0);
+   gtk_container_add (GTK_CONTAINER (scrolled_window), GTK_WIDGET (view));
+   gtk_widget_show_all (scrolled_window);
 
-   main_win_scrollbar = (GtkWidget *)gtk_vscrollbar_new (GTK_ADJUSTMENT(adj));
-   gtk_range_set_update_policy (GTK_RANGE (main_win_scrollbar), GTK_UPDATE_CONTINUOUS);
+   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
+   gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
-   gtk_table_attach (GTK_TABLE (table), main_win_scrollbar, 1, 2, 0, 1,
-		     (GtkAttachOptions) (0),
-		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
-   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
-		       (GtkSignalFunc) MainWinScrolled,
-		       (gpointer) main_win_scrollbar);       
-   gtk_widget_show (main_win_scrollbar);  
-
-
-   gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
-   
-   /*  Install event handlers */
-   gtk_signal_connect (GTK_OBJECT (canvas), "expose_event",
-		       GTK_SIGNAL_FUNC (log_repaint), NULL);
-   gtk_signal_connect (GTK_OBJECT (app), "key_press_event",
-		       GTK_SIGNAL_FUNC (HandleLogKeyboard), NULL);
-   gtk_signal_connect (GTK_OBJECT (canvas), "button_press_event",
-		       GTK_SIGNAL_FUNC (handle_log_mouse_button), NULL);
-   g_signal_connect (G_OBJECT (canvas), "scroll_event", 
-   		     G_CALLBACK (handle_log_mouse_scroll), NULL);
-   gtk_signal_connect (GTK_OBJECT (canvas), "size_allocate",
-		       GTK_SIGNAL_FUNC (CanvasResized), NULL);
-   gtk_widget_set_events (canvas, GDK_EXPOSURE_MASK |
-			  GDK_BUTTON_PRESS_MASK |
-			  GDK_POINTER_MOTION_MASK |
-			  GDK_SCROLL_MASK
-			  );
-
-   gtk_widget_set_events (app, GDK_KEY_PRESS_MASK);
-
-
-   gtk_widget_set_style (canvas, cfg->white_bg_style);
-   gtk_widget_show (canvas);
+   /* Add signal handlers */
+   g_signal_connect (G_OBJECT (selection), "changed",
+                     G_CALLBACK (handle_selection_changed_cb), NULL);
 
+   g_signal_connect (G_OBJECT (view), "row_activated",
+                     G_CALLBACK (handle_row_activation_cb), NULL);
 
    /* Create status area at bottom */
    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width ( GTK_CONTAINER (hbox), 3);
 
-   label = (GtkLabel *)gtk_label_new (_("Filename: "));
-   gtk_label_set_justify (label, GTK_JUSTIFY_LEFT);
-   gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (label), FALSE, FALSE, 0);
-   gtk_widget_show (GTK_WIDGET (label));  
-
-   filename_label = (GtkLabel *)gtk_label_new ("");
-   gtk_widget_show ( GTK_WIDGET (filename_label));  
-   gtk_label_set_justify (label, GTK_JUSTIFY_LEFT);
-   gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (filename_label), 
-		       FALSE, FALSE, 0);
-   
    hbox_date = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width ( GTK_CONTAINER (hbox_date), 3);
 
@@ -487,170 +488,13 @@ CreateMainWin ()
 
 }
 
-/* ----------------------------------------------------------------------
-   NAME:          MainScreenResized
-   DESCRIPTION:   The main screen was resized.
-   ---------------------------------------------------------------------- */
-
-void
-CanvasResized (GtkWidget *widget, GtkAllocation *allocation)
-{
-  DB (
-      if (allocation)
-      	printf ("Main screen resized!\n New size = (%d,%d)\n", 
-		allocation->width, allocation->height);
-      else
-        printf ("Main screen resized!\nNo allocation :(\n");
-	);
-}
-
-/* ----------------------------------------------------------------------
-   NAME:          ScrollWin
-   DESCRIPTION:   When the mouse button is released we scroll the window.
-   ---------------------------------------------------------------------- */
-
-gboolean
-ScrollWin (GtkRange *range, gpointer event)
-{
-  int newln;
-  
-  newln = (int) range->adjustment->value;
-  if (curlog == NULL ||
-      newln >= curlog->lstats.numlines ||
-      newln < 0)
-    return FALSE;
-
-  /* evil, yes */
-  if (newln == 0)
-	  newln = 1;
-  
-  /* Goto mark */
-  MoveToMark (curlog);
-  curlog->firstline = 0;
-  
-  ScrollDown (newln - curlog->curmark->ln);
-  
-  /* Repaint screen */
-  log_repaint(NULL, NULL);
-
-  return FALSE;
-}
-
-/* ----------------------------------------------------------------------
-   NAME:          MainWinScrolled
-   DESCRIPTION:   main window scrolled
-   ---------------------------------------------------------------------- */
-
-void
-MainWinScrolled (GtkAdjustment *adjustment, GtkRange *range)
-{
-  int newln, howmuch;
-  DateMark *mark;
-
-  newln = (int) range->adjustment->value;
-
- if (newln < 0 ||
-     curlog == NULL)
-   return;
-
- /* evil, yes */
- if (newln == 0)
-	 newln = 1;
-
- if (newln >= curlog->lstats.numlines)
-   newln = curlog->lstats.numlines - 1;
-
-  /* Find mark which has this line */
-  mark = curlog->curmark;
-  if (mark == NULL)
-    return;
-
-  if (newln >= mark->ln)
-    {
-      while( mark->next != NULL)
-	{
-	  if (newln <= mark->next->ln)
-	    break;
-	  mark = mark->next;
-	}
-    }
-  else
-    {
-      while( mark->prev != NULL)
-	{
-	  if (newln >= mark->ln)
-	    break;
-	  mark = mark->prev;
-	}
-    }
-
-  /* Now lets make it the current mark */
-  cursor_visible = FALSE;
-  howmuch = newln - curlog->ln;
-  curlog->ln = newln;
-  if (mark != curlog->curmark)
-    {
-      curlog->curmark = mark;
-      MoveToMark (curlog);
-      curlog->firstline = 0;
-      howmuch = newln - mark->ln;
-    }
-
-  /* Update status area */
-  UpdateStatusArea ();
-
-  /* Only scroll when the scrollbar is released */
-  if (howmuch > 0)
-      ScrollDown (howmuch);
-  else
-      ScrollUp (-1*howmuch);
-
-  if (howmuch != 0)
-    log_repaint(NULL, NULL);
-}
-
-
-/* ----------------------------------------------------------------------
-   NAME:          set_scrollbar_size
-   DESCRIPTION:   Set size of scrollbar acording to file.
-   ---------------------------------------------------------------------- */
-
-void set_scrollbar_size (int num_lines)
-{
-  GtkObject *adj;
-
-  adj = gtk_adjustment_new (-1, 0.0, num_lines,
-			    1.0, 10.0, (float) LINES_P_PAGE);
-  gtk_range_set_adjustment (GTK_RANGE (main_win_scrollbar),
-			    GTK_ADJUSTMENT (adj));
-  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
-		      (GtkSignalFunc) MainWinScrolled,
-		      (gpointer) main_win_scrollbar);       
-  if (curlog != NULL )
-  {
-  	gtk_adjustment_set_value (GTK_ADJUSTMENT(adj), curlog->ln); 
-	adj = (GtkObject*)gtk_viewport_get_hadjustment (GTK_VIEWPORT (viewport));
-  	gtk_adjustment_set_value (GTK_ADJUSTMENT(adj),0); 
-  }
-  else
-	adj = gtk_adjustment_new (100.0, 0.0, 101.0, 1, 10, 101.0);
-
-  gtk_range_set_adjustment (GTK_RANGE (main_win_hor_scrollbar),GTK_ADJUSTMENT (adj));
-
-  gtk_widget_realize (main_win_scrollbar);
-  gtk_widget_realize (main_win_hor_scrollbar);
-
-  gtk_widget_queue_resize (main_win_scrollbar);
-  gtk_widget_queue_resize (main_win_hor_scrollbar);
-}
-
 /* ----------------------------------------------------------------------
    NAME:          CloseLogMenu
    DESCRIPTION:   Close the current log.
    ---------------------------------------------------------------------- */
 
 void
-CloseLogMenu (GtkWidget * widget, gpointer user_data)
+CloseLogMenu (GtkWidget *widget, gpointer user_data)
 {
    int i;
 
@@ -658,18 +502,20 @@ CloseLogMenu (GtkWidget * widget, gpointer user_data)
       return;
 
    CloseLog (curlog);
+
    numlogs--;
    if (numlogs == 0)
    {
       curlog = NULL;
       loglist[0] = NULL;
       curlognum = 0;
-      log_repaint (NULL, NULL);
+      log_repaint ();
       if (loginfovisible)
-	 RepaintLogInfo (NULL, NULL);
-      set_scrollbar_size (1);
-      gtk_widget_set_sensitive (log_menu[7].widget, FALSE); 
-      gtk_widget_set_sensitive (log_menu[4].widget, FALSE); 
+	      RepaintLogInfo ();
+      gtk_widget_set_sensitive (log_menu[MENU_PROPERTIES].widget, FALSE); 
+      gtk_widget_set_sensitive (log_menu[MENU_CLOSE].widget, FALSE); 
+      gtk_widget_set_sensitive (log_menu[MENU_CLOSE_ALL].widget, FALSE); 
+      gtk_widget_set_sensitive (log_menu[MENU_MONITOR].widget, FALSE); 
       for ( i = 0; i < 3; i++) 
          gtk_widget_set_sensitive (view_menu[i].widget, FALSE); 
       return;
@@ -681,16 +527,15 @@ CloseLogMenu (GtkWidget * widget, gpointer user_data)
    if (curlognum > 0)
       curlognum--;
    curlog = loglist[curlognum];
-   log_repaint (NULL, NULL);
+   log_repaint ();
 
    if (loginfovisible)
-      RepaintLogInfo (NULL, NULL);
+      RepaintLogInfo ();
 
    /* Change menu entry if there is only one log */
    if (numlogs < 2)
-     gtk_widget_set_state (log_menu[3].widget, GTK_STATE_INSENSITIVE);
+     gtk_widget_set_state (log_menu[MENU_SWITCH_LOG].widget, GTK_STATE_INSENSITIVE);
 
-   set_scrollbar_size (curlog->lstats.numlines);
 }
 
 /* ----------------------------------------------------------------------
@@ -702,6 +547,7 @@ void
 change_log_menu (GtkWidget * widget, gpointer user_data)
 {
   change_log (1);
+
 }
 
 /* ----------------------------------------------------------------------
@@ -738,36 +584,45 @@ FileSelectOk (GtkWidget * w, GtkFileSelection * fs)
    }      
 
    if (f != NULL) {
-      if ((tl = OpenLogFile (f)) != NULL)
-      {
-	 curlog = tl;
-	 loglist[numlogs] = tl;
-	 numlogs++;
-	 curlognum = numlogs - 1;
+       if ((tl = OpenLogFile (f)) != NULL) {
+           if (numlogs > 0) {
+               memset (curlog->expand_paths, 0, 
+                       sizeof(curlog->expand_paths));
+               save_rows_to_expand (curlog); 
+           }
+           if (!numlogs) 
+               gtk_widget_set_sensitive (log_menu[MENU_CLOSE].widget, TRUE); 
+           
+	       curlog = tl;
+	       curlog->first_time = TRUE; 
+	       curlog->mon_on = FALSE;
+		   loglist[numlogs] = tl;
+		   numlogs++;
+		   curlognum = numlogs - 1;
 
-	 /* Clear window */
-	 log_repaint (NULL, NULL);
-	 if (loginfovisible)
-	   RepaintLogInfo (NULL, NULL);
-	 if (calendarvisible)
-	   init_calendar_data();
-	 UpdateStatusArea();
+		   /* Clear window */
+		   log_repaint ();
+		   if (loginfovisible)
+			   RepaintLogInfo ();
+		   if (calendarvisible)
+			   init_calendar_data();
 
-	 /* Set main scrollbar */
-	 set_scrollbar_size (curlog->lstats.numlines);
+	       UpdateStatusArea();
 
-	 if (numlogs)
-	 {
-	   int i;
-	   if (numlogs >= 2)
-	     gtk_widget_set_sensitive (log_menu[3].widget, TRUE);
-	   gtk_widget_set_sensitive (log_menu[7].widget, TRUE);
-	   gtk_widget_set_sensitive (log_menu[4].widget, TRUE);
-	   for ( i = 0; i < 3; i++) 
-	     gtk_widget_set_sensitive (view_menu[i].widget, TRUE);
-	 } 
-      }
+		   if (numlogs) {
+			   int i;
+			   if (numlogs >= 2)
+			       gtk_widget_set_sensitive (log_menu[MENU_SWITCH_LOG].widget, TRUE);
+
+			   gtk_widget_set_sensitive (log_menu[MENU_CLOSE_ALL].widget, TRUE);
+			   gtk_widget_set_sensitive (log_menu[MENU_PROPERTIES].widget, TRUE);
+			   gtk_widget_set_sensitive (log_menu[MENU_MONITOR].widget, TRUE);
+			   for (i = 0; i < 3; ++i) 
+			       gtk_widget_set_sensitive (view_menu[i].widget, TRUE);
+		   } 
+       }
    }
+
    g_free (f);
 
 }
@@ -823,12 +678,9 @@ LoadLogMenu (GtkWidget * widget, gpointer user_data)
    gtk_widget_show (filesel);
 
    open_file_dialog = filesel;
+
 }
 
-
-
-
-
 /* ----------------------------------------------------------------------
    NAME:          ExitProg
    DESCRIPTION:   Callback to call when program exits.
@@ -838,6 +690,43 @@ void
 ExitProg (GtkWidget * widget, gpointer user_data)
 {
    CloseApp ();
+
+}
+
+/* ----------------------------------------------------------------------
+   NAME:          CloseAllLogs 
+   DESCRIPTION:   Close all log files
+   ---------------------------------------------------------------------- */
+
+void 
+CloseAllLogs (void)
+{
+   int i;
+   
+   if (numlogs == 0)
+      return;
+
+   for (i = 0; i < numlogs; ++i) 
+       CloseLog (loglist[i]);
+
+   numlogs = 0;
+   curlognum = 0;
+   curlog = NULL;
+   loglist[0] = NULL;
+
+   log_repaint ();
+   if (loginfovisible)
+	  RepaintLogInfo ();
+
+   gtk_widget_set_sensitive (log_menu[MENU_MONITOR].widget, FALSE); 
+   gtk_widget_set_sensitive (log_menu[MENU_PROPERTIES].widget, FALSE); 
+   gtk_widget_set_sensitive (log_menu[MENU_CLOSE].widget, FALSE); 
+   gtk_widget_set_sensitive (log_menu[MENU_CLOSE_ALL].widget, FALSE); 
+   for ( i = 0; i < 3; i++) 
+       gtk_widget_set_sensitive (view_menu[i].widget, FALSE); 
+
+   gtk_widget_set_state (log_menu[MENU_SWITCH_LOG].widget, GTK_STATE_INSENSITIVE); 
+
 }
 
 /* ----------------------------------------------------------------------
@@ -855,12 +744,8 @@ CloseApp (void)
 
    numlogs = 0;
 
-   if (G_IS_OBJECT (log_layout)) {
-      g_object_unref (G_OBJECT (log_layout));
-      log_layout = NULL;
-   }
-
    gtk_main_quit ();   
+
 }
 
 /* ----------------------------------------------------------------------
@@ -957,6 +842,7 @@ open_databases (void)
 
 	/* If debugging then print DB */
 	DB (print_db (regexp_db));
+
 }
 
 /* ----------------------------------------------------------------------
@@ -970,6 +856,7 @@ IsLeapYear (int year)
       return TRUE;
    else
       return FALSE;
+
 }
 
 void SetDefaultUserPrefs(UserPrefsStruct *prefs)
@@ -979,15 +866,8 @@ void SetDefaultUserPrefs(UserPrefsStruct *prefs)
 	gchar *logfile = NULL;
 	struct stat filestat;
 	
-	/* FIXME: load saved values. Should install gconf schemas file for
-	** defaults 
-	*/
-	prefs->process_column_width = 12;
-	prefs->hostname_column_width = 15;
-	prefs->message_column_width = 24;
-	
 	logfile = gconf_client_get_string (client, "/apps/logview/logfile", NULL);
-	if (logfile != NULL) {
+	if (logfile != NULL && strcmp (logfile, "")) {
 		prefs->logfile = g_strdup (logfile);
 		g_free (logfile);
 	}
@@ -997,8 +877,6 @@ void SetDefaultUserPrefs(UserPrefsStruct *prefs)
 			prefs->logfile = g_strdup ("/var/adm/messages");
 		else if (lstat("/var/log/messages", &filestat) == 0) 
 			prefs->logfile = g_strdup ("/var/log/messages");
-		else if (lstat("/var/log/sys.log", &filestat) == 0) 
-			prefs->logfile = g_strdup ("/var/log/sys.log");
 		else
 			prefs->logfile = NULL;
 	}
@@ -1008,13 +886,6 @@ void SetDefaultUserPrefs(UserPrefsStruct *prefs)
 void SaveUserPrefs(UserPrefsStruct *prefs)
 {
     gconf_client_set_string (client, "/apps/logview/logfile", prefs->logfile, NULL);
-    gconf_client_set_int (client, "/apps/logview/process_column_width",
-    			  prefs->process_column_width, NULL);
-    gconf_client_set_int (client, "/apps/logview/hostname_column_width",
-    			  prefs->hostname_column_width, NULL);
-    gconf_client_set_int (client, "/apps/logview/message_column_width",
-    			  prefs->message_column_width, NULL);
-
 }
 
 static void 
@@ -1035,5 +906,12 @@ toggle_zoom (void)
 	close_zoom_view (app, NULL);
     }
     else
-	create_zoom_view (NULL, NULL);
+	create_zoom_view ();
+
+}
+
+static void 
+toggle_collapse_rows (void)
+{
+    gtk_tree_view_collapse_all (GTK_TREE_VIEW (view));
 }
